from pprint import pprint
import yaml
from tabulate import tabulate
from funcy import project
import emoji
import os
import textwrap
class Ls:
    def __init__(self):
        super().__init__()

    def process(self, args):
        """ opens the warp database and print its content """
        with open(args['workspace_warp_database'],'r') as f:
            data = yaml.load(f, Loader=yaml.FullLoader)
            if not data:
                data = []


        # when ls, verify which dir exists and mark import emoji

        if args['alias']:
            data = [ d for d in data if any( [ a for a in args['alias'] if a.upper() in d['alias'].upper() ]  )    ]


        if not args['verbose']:
            data = [ project(d,['alias', 'local', 'remote' ]) for d in data] 

        for item in data:

            # mark items that doesnt exists
            # src
            if not os.path.exists(item['local']):
                if args['debug']: 
                    print(f"Source path '{item['local']}'' does not exist yet, use 'sync --down --force' to get it.")
                item['src'] = emoji.emojize(item['local'] + ":boom:", use_aliases = True)
            # break long paths into lines
            # item['src'] = '\n\t'.join(textwrap.wrap(text = item['src'], width = 40))

            # dst
            for i in range(len(item['remote'])):
                d = item['remote'][i]
                if not os.path.exists(d):
                    if args['debug']: 
                        print(f"Destination path '{d}'' does not exist yet, use 'sync up --force' to push local to remote.")
                    d = emoji.emojize(d + ":boom:", use_aliases = True)
                # d = '\n\t'.join(textwrap.wrap(text= d, width=40))
                item['remote'][i] = d
        
            # flatten list to better print it
            item['remote'] = '\n'.join(item['remote'])


        if 'table' in args['format']:
            print(tabulate(data, headers="keys", tablefmt = "grid"))
        elif 'tree' in args['format']:
            for item in data:
                print(item['alias'])
                print(f"    local: {item['local']}")
                print(f"    remote: {item['remote']}")
        else:
            raise Exception("Unknown format")
        
   
   


import yaml
from os import path
from os.path import expanduser
import pathlib
from plumbum import local, FG, BG, TF, RETCODE
from plumbum.cmd import rsync
from pprint import pprint
from funcy import project
from wws.commands import utils
import time
class Sync:
    def __init__(self):
        super().__init__()

    def process(self, args):
        getattr(self, args['cmd_sync'])(args)

    def up(self, args):
        with open(args['workspace_warp_database'],'r') as f:
            data = yaml.load(f, Loader=yaml.FullLoader)
            if not data:
                data = []


        # filter only the selected aliases
        if args['alias']:
            data = [ d for d in data if any( [ a for a in args['alias'] if a.upper() in d['alias'].upper() ])]

        start = time.time()
        # synchronize warp points
        for item in data:

            if not path.exists(item['local']):
                if args['debug'] or args['verbose']:
                    print(f"Source path '{item['local']}' does not exits, skipping.")
                    continue

            if args['debug'] or args['verbose']:
                print(f"synchronizing {item['local']}")

            for dst in item['remote']:
                # this approach need rework for remote locations (rsync over ssh)...
                if not path.exists(dst): 
                    if args['force']:
                        print("Force creating remote directory")
                        if not args['dry_run']:
                            pathlib.Path(dst).mkdir(parents=True, exist_ok=True)    
                    else:
                        if args['debug'] or args['verbose']:
                            print(f"Destination path '{dst}' does not exits, skipping.")
                        continue

                params = []
                if args["dry_run"]:
                    params.append("--dry-run")

                # include global patterns
                for ex in args['exclude_patterns']:
                    params.append(f"--exclude={ex}")

                # include per workspace options
                if item['opts']:
                    params.extend(item['opts'])

                # fix source and destination
                if item['local'][-1] is not '/':
                    item['local'] += '/'
                if dst[-1] is not '/':
                    dst += '/'
                

                # finish with source -> dest
                params.append(item['local'])
                params.append(dst)
                if args['debug']:
                    pprint(params)

                if args["verbose"]: # for safety always dry run
                    print(f"\tto: {dst}")
                
                out = rsync[params].run() 
                if args["debug"] or args['dry_run']: 
                    pprint(out) 
        
        end = time.time()
        from plumbum.cmd import osascript
        osascript['-e',f'display notification "[WWS] Workspace synchronized in {int(end - start)} seconds!"'].run()


 


    def down(self, args):
        with open(args['workspace_warp_database'],'r') as f:
            data = yaml.load(f, Loader=yaml.FullLoader)
            if not data:
                data = []


        # filter only the selected aliases
        if args['alias']:
            data = [ d for d in data if any( [ a for a in args['alias'] if a.upper() in d['alias'].upper() ])]

        start = time.time()
        # synchronize warp points
        for item in data:

            if not path.exists(item['local']):
                if args['force']:
                    print("Force creating local directory")
                    if not args['dry_run']:
                        pathlib.Path(item['local']).mkdir(parents=True, exist_ok=True)    
                else:
                    if args['debug'] or args['verbose']:
                        print(f"Local path '{item['local']}' does not exits, skipping.")
                    continue


            if args['debug'] or args['verbose']:
                print(f"synchronizing {item['local']}")

            # select the first remote path
            dst = item['remote'][0]

            # this approach need rework for remote locations (rsync over ssh)...
            if not path.exists(dst): 
                if args['debug'] or args['verbose']:
                    print(f"Remote path '{dst}' does not exits, skipping.")
                continue

            params = []
            if args["dry_run"]:
                params.append("--dry-run")

            # include global patterns
            for ex in args['exclude_patterns']:
                params.append(f"--exclude={ex}")

            # include per workspace options
            if item['opts']:
                params.extend(item['opts'])

            # fix source and destination
            if item['local'][-1] is not '/':
                item['local'] += '/'
            if dst[-1] is not '/':
                dst += '/'
            

            # finish with reversing direction remote -> local
            params.append(dst)
            params.append(item['local'])
            if args['debug']:
                pprint(params)

            if args["verbose"]: # for safety always dry run
                print(f"\tto: {dst}")
            
            out = rsync[params].run() 
            if args["debug"] or args['dry_run']: 
                pprint(out) 

        end = time.time()
        from plumbum.cmd import osascript
        osascript['-e',f'display notification "[WWS] Workspace synchronized in {int(end - start)} seconds!"'].run()



from pprint import pprint
import yaml
from tabulate import tabulate
from funcy import project
from wws.commands import utils

class Rm:
    def __init__(self):
        super().__init__()

    def process(self, args):
        """ edits the warp database  """
        if args['debug']:
            pprint(args)
        with open(args['workspace_warp_database'],'r+') as f:
            data = yaml.load(f, Loader=yaml.FullLoader)
            if not data:
                data = []

        
            remove_entries = dict()
            keep_entries = dict()
        
            remove_entries = [ d for d in data if any( [ a for a in args['alias'] if a.upper() in d['alias'].upper() ]  )  ]
            keep_entries = [ d for d in data if not any( [ a for a in args['alias'] if a.upper() in d['alias'].upper() ]  )  ]


            
            if not args['verbose']:
                data = [ project(d,['alias', 'local', 'remote' ]) for d in remove_entries] 
            else:
                data = remove_entries

            if not data:
                print("Nothing to remove.")
                exit()
            
            print("Entries to remove:")
            print(tabulate(data, headers="keys", tablefmt = "psql"))
            rm = utils._confirm("Are you sure to remove these entries?")

            if rm:
                f.seek(0)
                f.truncate()
                yaml.dump(keep_entries, f, default_flow_style=False)

            print("Aliases were removed but data remain untouched. Please remove the listed source directories.")


            


#! env python3
import sys
import os
from pprint import pprint
from funcy import merge
from wws.commands import utils

def agent(args):
    """  
        Synchronization agent command dispatcher
    """
    from wws.commands import agent
    if ['verbose']:
        print("agent command invoked")
    cmd = agent.Agent()
    cmd.process(args)

def cmd_agent(subparsers):
    subcommand = 'cmd_agent'
    cmd_parser = subparsers.add_parser('agent', help='Workspace synchronization agent management')
    action_cmd_parser = cmd_parser.add_mutually_exclusive_group(required=True)
    action_cmd_parser.add_argument('-s','--start',     action='store_const', const='start', dest=subcommand, help='Start the synchronization agent')
    action_cmd_parser.add_argument('-p','--stop',      action='store_const', const='stop', dest=subcommand, help='Stop the synchronization agent')
    action_cmd_parser.add_argument('-r','--reload',    action='store_const', const='reload', dest=subcommand, help='Reload the synchronization agent')
    action_cmd_parser.add_argument('-c','--configure', action='store_const', const='configure', dest=subcommand, help='Configure the synchronization agent')
    action_cmd_parser.add_argument('-u','--status', action='store_const', const='status', dest=subcommand, help='Configure the synchronization agent')
    cmd_parser.add_argument("--timer", "-t", default=1200, type=int  ,required=False)

# --------------------- --------------------- --------------------- --------------------- ---------------------

def ls(args):
    """  
        List workspace command dispatcher
    """
    from wws.commands import ls
    if ['verbose']:
        print("ls command invoked")
    cmd = ls.Ls()
    cmd.process(args)
 
def cmd_ls(subparsers):
    # ls command
    subcommand = 'cmd_ls'
    cmd_parser = subparsers.add_parser('ls', help='List synchronized workspaces')
    cmd_parser.add_argument("-a","--alias", nargs='+', required=False, help='specify the item or items to list')
    cmd_parser.add_argument("-o","--with-options", action = 'store_true', required=False, help='print detailed options')

    action_cmd_parser = cmd_parser.add_mutually_exclusive_group(required=False)
    action_cmd_parser.add_argument('-e','--tree', action='store_const', const = "tree", default = 'tree', dest='format', help='print output in tree format')
    action_cmd_parser.add_argument('-t','--table', action='store_const', const = "table", dest='format', help='print output in table format')

# --------------------- --------------------- --------------------- --------------------- ---------------------
def add(args):
    """  
        Add workspace command dispatcher
    """

    from wws.commands import add
    if ['verbose']:
        print("Add command invoked")
    cmd = add.Add()
    cmd.process(args)
 
def cmd_add(subparsers):
    # add command
    cmd_parser = subparsers.add_parser('add', help='Add synchronized workspaces')
    cmd_parser.add_argument("-a","--alias", type=str, required=False, help='Specify an alias for the warp point')
    cmd_parser.add_argument("-l","--local", type=str, required=True, help='Specify local path')
    cmd_parser.add_argument("-r","--remote",nargs='+', required=True, help='Specify remote path')


# --------------------- --------------------- --------------------- --------------------- ---------------------
def rm(args):
    """  
        Remove workspace command dispatcher
    """
    from wws.commands import rm
    if ['verbose']:
        print("rm command invoked")
    cmd = rm.Rm()
    cmd.process(args)
 
def cmd_rm(subparsers):
    # ls command
    subcommand = 'cmd_rm'
    cmd_parser = subparsers.add_parser('rm', help='Remove synchronized workspaces')
    cmd_parser.add_argument("-a","--alias", nargs="+", required=True, help='Specify an alias for the warp point')
    # cmd_parser.add_argument("-s","--src", "--source", type=str, required=False, help='Specify source path')
    # cmd_parser.add_argument("-d","--dst", "--destination", nargs='+', required=False, help='Specify destination path')

# --------------------- --------------------- --------------------- --------------------- ---------------------
def sync(args):
    """  
        Sync workspace command dispatcher
    """
    from wws.commands import sync
    if ['verbose']:
        print("Sync command invoked")
    cmd = sync.Sync()
    cmd.process(args)
    
 
def cmd_sync(subparsers):
    # sync command
    subcommand = 'cmd_sync'
    cmd_parser = subparsers.add_parser('sync', help='Sync command to force  operations on workspaces')
    cmd_parser.add_argument("--dry-run", action='store_true', required=False, help='Non destructive operation, don\'t actually copies anything')
    cmd_parser.add_argument("-a","--alias", nargs='+', required=False, help='specify the item or items to selective sync')
    cmd_parser.add_argument('--force', action='store_true', help='force synchronize operations creating dirs when necessary')
    action_cmd_parser = cmd_parser.add_mutually_exclusive_group(required=True)
    action_cmd_parser.add_argument('--up', action='store_const', const='up', dest=subcommand, help='Copy data from local to remote')
    action_cmd_parser.add_argument('--down', action='store_const', const='down', dest=subcommand, help='Copy data from remote to local')


# --------------------- --------------------- --------------------- --------------------- ---------------------

def main():
    """ Workspace warp and sync utility. 
    This application helps you to keep local directories in synch with
    remote locations, usually cloud drives.
    """
    import argparse
    main_parser = argparse.ArgumentParser()
    main_parser.add_argument('-v', '--verbose', action = 'store_true', help = "Make the command verbose")
    main_parser.add_argument( "-c", "--config", default = "~/.wws/settings.yaml", help = "Set the configuration file", type=str  ,required=False)
    main_parser.add_argument( "-w", "--warp-database", default = "~/.wws/data.yaml", dest='workspace_warp_database', help = "Set workspace warp file", type=str  ,required=False)
    main_parser.add_argument( "-g", "--debug", dest='debug', action='store_true', required=False)

    # define commands 
    subparsers = main_parser.add_subparsers(dest='cmd_main', help="Available Workspace Warp and Sync commands")
    subparsers.required = True
    cmd_add(subparsers)
    cmd_rm(subparsers)
    cmd_ls(subparsers)
    cmd_sync(subparsers)
    cmd_agent(subparsers)

    # flatten arguments and invoke command dispatcher
    args = vars(main_parser.parse_args())
    
    # initialize settings
    args['config'] = os.path.expanduser(args['config'])
    # if the file exists
    if not os.path.exists(args['config']):
        if not utils._confirm(f"Workspace Warp settings not found, initialize an empty one at {args['config']}?"):
            print("Database initialization aborted. WWS was not configured.")
            exit()
        # if the directory of the file exists
        if not os.path.exists(os.path.dirname(args['config'])):
            if args['debug']:
                print("Creating configuration directory: %s " ,os.path.dirname(args['config']))
            # create the dir
            os.mkdir(os.path.dirname(args['config']))
        # create the file
        utils.init(args)

    # load settings
    settings = utils.load_settings(args['config'])
    
    # coalesce a list of dicts into a single dict
    default_options = dict()
    for conf in settings:
        default_options =  merge(default_options, conf)
        
    # join settings overriding the default for command line  
    args = merge(default_options, args)
    
    args['workspace_warp_database'] = os.path.expanduser(args['workspace_warp_database'])
    if not os.path.exists(args['workspace_warp_database']):
        if not utils._confirm(f"Warp database not found, initialize an empty one at {args['workspace_warp_database']}?"):
            print("Database initialization aborted")
            exit()

        if not os.path.exists(os.path.dirname(args['workspace_warp_database'])):
            if args['debug']:
                print("Creating configuration directory: %s " ,os.path.dirname(args['workspace_warp_database']))
            # create the dir
            os.mkdir(os.path.dirname(args['workspace_warp_database']))
        # just create an emtpy file
        with open(args['workspace_warp_database'],'w+') as f:
            pass
    else:
        utils.migrate_database(args)

    if args['debug'] or args['verbose']:
        pprint(args)

    # make sure binaries we use exists (rsync and rclone in the future)

    # invoke command
    getattr(sys.modules[__name__], args['cmd_main'])(args)
# --------------------- --------------------- --------------------- --------------------- ---------------------


if __name__ == '__main__':
    main()



from pprint import pprint
from os.path import basename, normpath, abspath
import yaml
class Add:
    def __init__(self):
        super().__init__()

    def process(self, args):
        """ edits the warp database  """
        if args['debug']:
            pprint(args)
        with open(args['workspace_warp_database'],'r+') as f:
            data = yaml.load(f, Loader=yaml.FullLoader)
            if not data:
                data = []
        
            # expand 
            args['local'] = abspath(args['local'])
            args['remote'] = [ abspath(a) for a in args['remote'] ]

            if not args['alias']:
                # add the last dir name as default
                args['alias'] = basename(normpath(args['local'])).lower()
                print(f"Since alias was not informed, suggesting one based on the source dir: {args['alias']}")

            if args['alias'] in [ a['alias'] for a in data ]:
                print("Alias already used. Choose another or remove it first")
                exit()

            entry = {
                "alias": args['alias'],
                "local": args['local'],
                "remote": args['remote'],
                "driver": "rsync",
                "opts": ["-avh", "--no-links" ,"--delete-after"],
            }
            data.append(entry)
            f.seek(0)
            f.truncate()
            yaml.dump(data, f, default_flow_style=False)
            # pprint(text)
            # f.write(text)




import yaml
from printy import printy, inputy
from os import path
from os.path import expanduser, basename, normpath, abspath
from pprint import pprint


def init(args):
    conf_path = args['config']
    settings = [ {"workspace_warp_database": args['workspace_warp_database'] }, {"exclude_patterns": ["Icon?",".DS_Store"]} ]
    with open(conf_path,'w+') as f:
        f.seek(0)
        f.truncate()
        yaml.dump(settings, f, default_flow_style=False)
    
    

def load_settings(conf_path):
    conf_path = expanduser(conf_path)

    if not path.exists(conf_path):
        print("Abort, configuration settings file not found.")
        exit()

    with open(conf_path,'r') as f:
        settings = yaml.load(f, Loader=yaml.FullLoader)
    return settings

def migrate_database(args):
    """ Updates the warp database  """
    if args['debug']:
        pprint(args)
    with open(args['workspace_warp_database'],'r+') as f:
        data = yaml.load(f, Loader=yaml.FullLoader)
        if not data:
            return
    
        pprint(data)
        for entry in data:
            if 'src' in entry:
                entry['local'] = entry['src']
                del entry['src']
            if 'dst' in entry:
                entry['remote'] = entry['dst']
                del entry['dst']

        pprint(data)
        # f.seek(0)
        # f.truncate()
        # yaml.dump(data, f, default_flow_style=False)

def _safe_print(msg, **kwargs):
    try:
        print(msg, **kwargs)
    except UnicodeEncodeError:
        print(msg.encode('ascii','ignore'), **kwargs)


def _info(msg, **kwargs):
    _safe_print(CCYAN + "¡" + CRESET + " " + msg, **kwargs)


def _local_with_brew_check(pkg):
    brew = local.get('brew','/usr/local/bin/brew')

    brew_has_pkg = brew['ls', '--versions', pkg].run(retcode=None)
    if brew_has_pkg[0] == 1:
        _info("Installing '" + pkg +"' terminal tool")
        if brew['install', pkg].run(retcode=None)[0] == 1:
            # return None if we fail to install
            return None

    return local.get(pkg, '/usr/local/bin/'+pkg)


import os
import sys

def get_script_path():
    return os.path.realpath(sys.argv[0])


# Prompt the yes/no-*question* to the user
# ref: https://stackoverflow.com/a/3041990/1700053
def _confirm(question, default='no'):
  from distutils.util import strtobool

  while True:
    try:
      return strtobool(inputy(question + " (y/n): ").lower())
    except ValueError:
      print("\tPlease use y/n or yes/no.\n")



import platform
import os
from os.path import expanduser
from pprint import pprint
from wws.commands.utils  import *
import re
from plumbum.cmd import launchctl

OSX_AGENT_CONF_PATH = '~/Library/LaunchAgents/com.porto.wws.plist'
OSX_AGENT_PROCESS_NAME = 'com.porto.wws'

class Agent:
    def __init__(self):
        super().__init__()

    def process(self, args):
        """ process the top level parameters and invoke subcommands 
        :param: args command line arguments
        :raise: Environment error when the os is not supported """
        getattr(self, args['cmd_agent'])(args)



    def configure(self, args):
        if args['verbose']:
            print("Command agent configure was called.")

        if args['verbose']:
            print(f"Configuring agent to fire at every {args['timer']} seconds.")

        if 'Darwin' in platform.system():
            _configure_osx_timer(args)
        elif 'Linux' in platform.system():
            _configure_linux_timer(args)
        else:
            raise EnvironmentError("Unsupported operating system")


    def start(self, args):
        global OSX_AGENT_CONF_PATH

        if 'Darwin' not in platform.system():
            raise EnvironmentError("Unsupported operating system - only Mac OSX is supported right now")

        OSX_AGENT_CONF_PATH = expanduser(OSX_AGENT_CONF_PATH)
        if not os.path.exists(OSX_AGENT_CONF_PATH):
            print("Agent is not configured, aborting")
            exit()


        if args['verbose']: 
            print("Deploying new agent")

        out = launchctl['load', '-w', OSX_AGENT_CONF_PATH].run()
        
        if any('already' in l for l in map(str,out)):
            print("Agent is already loaded.")
        else:
            print("Agent started.")



    def stop(self, args):
        global OSX_AGENT_CONF_PATH
        OSX_AGENT_CONF_PATH = expanduser(OSX_AGENT_CONF_PATH)

        if 'Darwin' not in platform.system():
            raise EnvironmentError("Unsupported operating system - only Mac OSX is supported right now")

        if not os.path.exists(OSX_AGENT_CONF_PATH):
            print("Agent is not configured, aborting")
            exit()

        if args['verbose']:
            print("Command agent stop was called.")
            print("Undeploying old agent")

        out = launchctl['unload', '-w', OSX_AGENT_CONF_PATH].run()
        if any('not find' in l for l in map(str,out)):
            print("Agent is already stopped.")
        else:
            print("Agent stopped.")



    def reload(self, args):
        if args['verbose']:
            print("Command agent reload was called.")
        self.stop(args)
        self.start(args)
        

    def status(self, args):
        global OSX_AGENT_CONF_PATH
        osxagent_file_path = expanduser(OSX_AGENT_CONF_PATH)


        if 'Darwin' not in platform.system():
            raise EnvironmentError("Unsupported operating system - only Mac OSX is supported right now")

        if args['verbose']:
            print("Command agent status was called.")

        out = launchctl['list'].run() 
        processes = [ i.split('\t')[2] for i in out[1].split('\n') if len(i.split('\t')) == 3 ]
        if OSX_AGENT_PROCESS_NAME in processes:
            print("Agent is running.")
            these_regex="<integer>(.+?)</integer>"
            pattern=re.compile(these_regex)

            with open(osxagent_file_path, 'r') as f:
                content = f.readlines()
                timerline = [ l for l in content if 'integer' in l ]
                timerline = re.findall(pattern, timerline[0])[0]
                print(f"It will fire periodically every {timerline} seconds.")

        else:
            print("Agent is not running.")




def _configure_osx_timer(args):
    global OSX_AGENT_CONF_PATH
    global OSX_AGENT_PROCESS_NAME

    pprint(args)
    script_path = get_script_path()
    configuration_file_path = expanduser(args['config'])
    database_file_path = expanduser(args['workspace_warp_database'])
    osxagent_file_path = expanduser(OSX_AGENT_CONF_PATH)

    seconds = int(args['timer'])

    text = f"""<?xml version="1.0" encoding="UTF-8"?>
    <!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
    <plist version="1.0">
    <dict>
        <key>Label</key>
        <string>{OSX_AGENT_PROCESS_NAME}</string>
        <key>ProgramArguments</key>
        <array>
            <string>/usr/local/bin/python3</string>
            <string>{script_path}</string>
            <string>-c</string>
            <string>{configuration_file_path}</string>
            <string>-w</string>
            <string>{database_file_path}</string>
            <string>sync</string>
            <string>--up</string>
        </array>
        <key>StandardErrorPath</key>
        <string>/tmp/wws.err</string>
        <key>StandardOutPath</key>
        <string>/tmp/wws.log</string>
        <key>StartInterval</key>
        <integer>{seconds}</integer>
    </dict>
    </plist>    """
        
    with open(osxagent_file_path, "+w") as f:
        f.writelines(text)
    print("Agent configured.")

def _configure_linux_timer(args):
    raise EnvironmentError("Unsupported OS")


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wws', 'wws.commands']

package_data = \
{'': ['*']}

install_requires = \
['emoji>=0.5.4,<0.6.0',
 'funcy>=1.14,<2.0',
 'plumbum>=1.6,<2.0',
 'pprint>=0.1,<0.2',
 'printy>=2.0.1,<3.0.0',
 'pyyaml>=5.3,<6.0',
 'tabulate>=0.8.7,<0.9.0']

entry_points = \
{'console_scripts': ['wws = wws.wws:main']}

setup_kwargs = {
    'name': 'wws',
    'version': '0.1.11',
    'description': 'A simple utility to synchronize local and remote paths',
    'long_description': '# workspace-warp-sync\nA simple utility to synchronize a local workspace with remote ones\n\n## Commands\n\n* ls \n* add   --src, --dst, --alias\n* rm    --src --dst --alias \n* agent --start, --stop, --configure, --reload, --status\n* sync  --up/down, --alias\n\n### Development\nConfigure visual studio code: \n* Install poetry on your system. \n* Install vscode with python extensions\n* Run poetry update\n* Run poetry shell\n* Run code .\n* select into vscode the python interpreter.',
    'author': 'Daniel Porto',
    'author_email': 'daniel.porto@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
